@@ -1,6 +1,7 @@
 package PHP::Functions::Password;
-# $Id: Password.pm,v 1.1 2017/06/13 00:51:28 cmanley Exp $
+# $Id: Password.pm,v 1.3 2017/06/21 19:33:52 cmanley Exp $
 use strict;
+use warnings;
 use Carp qw(croak);
 use Crypt::Eksblowfish ();
 use Crypt::OpenSSL::Random ();
@@ -11,6 +12,7 @@ our @EXPORT = ();
 our @EXPORT_OK = qw(
 	password_get_info
 	password_hash
+	password_needs_rehash
 	password_verify
 	PASSWORD_BCRYPT
 	PASSWORD_DEFAULT
@@ -21,11 +23,14 @@ our %EXPORT_TAGS = (
 	'consts'	=> [ grep /^PASSWORD_/, @EXPORT_OK ],
 	'funcs'		=> [ grep /^password_/, @EXPORT_OK ],
 );
-our $VERSION = sprintf '%d.%02d', q{$Revision: 1.1 $} =~ m/ (\d+) \. (\d+) /xg;
+our $VERSION = sprintf '%d.%02d', q{$Revision: 1.3 $} =~ m/ (\d+) \. (\d+) /xg;
 
 use constant PASSWORD_BCRYPT => 1;
 use constant PASSWORD_DEFAULT => PASSWORD_BCRYPT;
 
+use constant COST_DEFAULT => 10;
+use constant SIG_BCRYPT => '2y';
+
 use constant RE_BCRYPT_SETTINGS => qr#^\$(2[abxy]?)\$([0-3]\d)\$([A-Za-z0-9+\\/\.]{22})#;	# intentionally unanchored at the end
 use constant RE_BCRYPT_STRING   => qr#^\$(2[abxy]?)\$([0-3]\d)\$([A-Za-z0-9+\\/\.]{22})(.+)$#;
 
@@ -38,6 +43,7 @@ PHP::Functions::Password - Perl ports of PHP password functions
 This module provides ported PHP password functions.
 This module only supports the bcrypt algorithm, as is the case with the equivalent PHP functions at the date of writing this.
 All functions may also be called as class methods and support inheritance too.
+See L<http://php.net/manual/en/ref.password.php> for detailed usage instructions.
 
 =head1 SYNOPSIS
 
@@ -69,6 +75,7 @@ The following names can be imported into the calling namespace by request:
 
 	password_get_info
 	password_hash
+	password_needs_rehash
 	password_verify
 	PASSWORD_BCRYPT
 	PASSWORD_DEFAULT
@@ -76,11 +83,11 @@ The following names can be imported into the calling namespace by request:
 	:consts	- the PASSWORD_* constants
 	:funcs	- the password_* functions
 
-=head1 API
+=head1 PHP COMPATIBLE AND EXPORTABLE FUNCTIONS
 
 =over
 
-=item get_info($crypted)
+=item password_get_info($crypted)
 
 The same as L<http://php.net/manual/en/function.password-get-info.php>
 with the exception that it returns the following additional keys in the result:
@@ -89,11 +96,11 @@ with the exception that it returns the following additional keys in the result:
 	salt
 	hash
 
-Returns either a hash or a hashref depending on the context.
+Returns a hash in array context, else a hashref.
 
 =cut
 
-sub get_info {
+sub password_get_info {
 	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
 	my $crypted = shift;
 	unless ($crypted =~ RE_BCRYPT_STRING) {
@@ -114,9 +121,9 @@ sub get_info {
 		'options'	=> {
 			'cost'	=> $cost,
 		},
-		'algoSig'	=> $type,
-		'salt'		=> $salt,
-		'hash'		=> $hash,
+		'algoSig'	=> $type,	# extra
+		'salt'		=> $salt,	# extra
+		'hash'		=> $hash,	# extra
 	);
 	return wantarray ? %result : \%result;
 }
@@ -124,72 +131,71 @@ sub get_info {
 
 
 
-
-=item password_get_info($crypted)
-
-Exportable alias of C<get_info($crypted)>.
-
-=cut
-
-sub password_get_info {
-	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
-	return $proto->get_info(@_);
-}
-
-
-
-
-
-=item hash($password, %options)
-
-The same as L<http://php.net/manual/en/function.password-hash.php> but without the $algo argument.
-The algorithm can be specified with the 'algo' option if necessary, since it is optional after all.
-
-=cut
-
-sub hash {
-	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
-	my $password = shift;
-	my %options = @_ && ref($_[0]) ? %{$_[0]} : @_;
-	my $algo = $options{'algo'} || PASSWORD_DEFAULT;
-	($algo == PASSWORD_BCRYPT) || croak("Unsupported algorithm $algo");
-	my $salt = $options{'salt'} || $proto->_en_base64(Crypt::OpenSSL::Random::random_bytes(16));
-	my $cost = $options{'cost'} || 10;
-    my $settings = '$2y$' . sprintf('%.2u', $cost) . '$' . $salt;
-    return $proto->_bcrypt($password, $settings);
-}
-
-
-
-
-
 =item password_hash($password, $algo, %options)
 
-Exportable proxy function for C<hash($password, %options)> with the difference that this function uses the same argument structure as the PHP equivalent.
+Similar to L<http://php.net/manual/en/function.password-hash.php>
+with difference that the $algo argument is optional and defaults to PASSWORD_DEFAULT.
 
 =cut
 
 sub password_hash {
 	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
 	my $password = shift;
-	my $algo = shift;;
+	my $algo = shift // PASSWORD_DEFAULT;
+	($algo == PASSWORD_BCRYPT) || croak("Unsupported algorithm $algo");
 	my %options = @_ && ref($_[0]) ? %{$_[0]} : @_;
-	if ($algo) {
-		$options{'algo'} = $algo;
-	}
-	return $proto->hash($password, %options);
+	my $salt = $options{'salt'} || $proto->_en_base64(Crypt::OpenSSL::Random::random_bytes(16));
+	my $cost = $options{'cost'} || COST_DEFAULT;
+    my $settings = '$' . SIG_BCRYPT . '$' . sprintf('%.2u', $cost) . '$' . $salt;
+    return $proto->_bcrypt($password, $settings);
 }
 
 
 
 
-=item verify($password, $crypted)
+=item password_needs_rehash($crypted, $algo, %options)
+
+The same as L<http://php.net/manual/en/function.password-needs-rehash.php>.
+
+=cut
+
+sub password_needs_rehash {
+	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+	my $crypted = shift;
+	my $algo = shift // PASSWORD_DEFAULT;
+	my %options = @_ && ref($_[0]) ? %{$_[0]} : @_;
+	my $cost = $options{'cost'} // COST_DEFAULT;
+	unless ($algo == PASSWORD_BCRYPT) {
+		$options{'debug'} && warn("Can't do anything with unknown algorithm: $algo");
+		return 0;
+	}
+	my %info = password_get_info($crypted);
+	unless ($info{'algo'} == $algo) {
+		$options{'debug'} && warn('Algorithms differ: ' . $info{'algo'} . "<>$algo");
+		return 1;
+	}
+	if (($algo == PASSWORD_BCRYPT) && ($info{'algoSig'} ne SIG_BCRYPT)) {
+		$options{'debug'} && warn('Algorithm signatures differ: ' . $info{'algoSig'} . ' vs ' . SIG_BCRYPT);
+		return 1;
+	}
+	unless (defined($info{'options'}->{'cost'}) && ($info{'options'}->{'cost'} == $cost)) {
+		$options{'debug'} && warn('Cost mismatch: ' . $info{'options'}->{'cost'} . "<>$cost");
+		return 1;
+	}
+	return 0;
+}
+
+
+
+
+
+=item password_verify($password, $crypted)
 
 The same as L<http://php.net/manual/en/function.password-verify.php>.
 
 =cut
 
-sub verify {
+sub password_verify {
 	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
 	my ($password, $crypted) = @_;
 	unless ($crypted =~ RE_BCRYPT_STRING) {
@@ -211,28 +217,86 @@ sub verify {
 	return ($new_crypt =~ RE_BCRYPT_STRING) && ($4 eq $hash);
 }
 
+=back
 
 
 
 
-=item password_verify($password, $hash)
 
-Exportable alias of C<verify($password, $crypted)>.
+
+=head1 SHORTENED ALIAS METHODS
+
+=over
+
+=item get_info($crypted)
+
+Alias of C<password_get_info($crypted)>.
 
 =cut
 
-sub password_verify {
+sub get_info {
 	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
-	return $proto->verify(@_);
+	return $proto->password_get_info(@_);
 }
 
 
 
 
+=item hash($password, %options)
+
+Proxy method for C<password_hash($password, $algo, $options)>.
+The difference is that this method allows does have an $algo argument,
+but instead allows the algorithm to be specified with the 'algo' option (in %options).
+
+=cut
+
+sub hash {
+	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+	my $password = shift;
+	my %options = @_ && ref($_[0]) ? %{$_[0]} : @_;
+	my $algo = $options{'algo'} || PASSWORD_DEFAULT;
+	delete($options{'algo'});
+	return $proto->password_hash($password, $algo, %options);
+}
+
+
+
+
+=item needs_rehash($crypted, $algo, %options)
+
+Alias of C<password_needs_rehash($crypted, $algo, %options)>.
+
+=cut
+
+sub needs_rehash {
+	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+	return $proto->password_needs_rehash(@_);
+}
+
+
+
+
+
+=item verify($password, $crypted)
+
+Alias of C<verify($password, $crypted)>.
+
+=cut
+
+sub verify {
+	my $proto = @_ && UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+	return $proto->password_verify(@_);
+}
+
 =back
 
 =cut
 
+
+
+
+
+
 # From Crypt::Eksblowfish::Bcrypt.
 # This is a version of C<crypt> (see L<perlfunc/crypt>) that implements the bcrypt algorithm.
 sub _bcrypt {
@@ -250,7 +314,7 @@ sub _bcrypt {
 			'salt'		=> $proto->_de_base64($salt_base64),
 		}
 	);
-	return '$2y$' . $cost . '$' . $salt_base64 . $proto->_en_base64($hash);
+	return '$' . SIG_BCRYPT . '$' . $cost . '$' . $salt_base64 . $proto->_en_base64($hash);
 }
 
 
